@@ -1,5 +1,6 @@
+from box import Box, BoxList
 from restfly.endpoint import APIEndpoint
-from box import BoxList
+
 from pyzscaler.utils import snake_to_camel
 
 
@@ -21,21 +22,21 @@ class FirewallPolicyAPI(APIEndpoint):
         "users",
     ]
 
-    def list_rules(self):
+    def list_rules(self) -> BoxList:
         """
         Returns a list of all firewall filter rules.
 
         Returns:
-            :obj:`list` of :obj:`dict`: The list of firewall filter rules
+            :obj:`BoxList`: The list of firewall filter rules
 
         Examples:
             >>> for rule in zia.firewall.list_rules():
             ...    pprint(rule)
 
         """
-        return self._get("firewallFilteringRules", box=BoxList)
+        return self._get("firewallFilteringRules")
 
-    def add_rule(self, name: str, action: str, **kwargs):
+    def add_rule(self, name: str, action: str, **kwargs) -> Box:
         """
         Adds a new firewall filter rule.
 
@@ -71,7 +72,7 @@ class FirewallPolicyAPI(APIEndpoint):
             users (list): The IDs for the users that this rule applies to.
 
         Returns:
-            :obj:`dict`: The new firewall filter rule resource record.
+            :obj:`Box`: The new firewall filter rule resource record.
 
         Examples:
             Add a rule to allow all traffic to Google DNS (admin ranking is enabled):
@@ -109,7 +110,7 @@ class FirewallPolicyAPI(APIEndpoint):
 
         return self._post("firewallFilteringRules", json=payload)
 
-    def get_rule(self, rule_id: str):
+    def get_rule(self, rule_id: str) -> Box:
         """
         Returns information for the specified firewall filter rule.
 
@@ -117,7 +118,7 @@ class FirewallPolicyAPI(APIEndpoint):
             rule_id (str): The unique identifier for the firewall filter rule.
 
         Returns:
-            :obj:`dict`: The resource record for the firewall filter rule.
+            :obj:`Box`: The resource record for the firewall filter rule.
 
         Examples:
             >>> pprint(zia.firewall.get_rule('431233'))
@@ -125,7 +126,7 @@ class FirewallPolicyAPI(APIEndpoint):
         """
         return self._get(f"firewallFilteringRules/{rule_id}")
 
-    def update_rule(self, rule_id: str, **kwargs):
+    def update_rule(self, rule_id: str, **kwargs) -> Box:
         """
         Updates an existing firewall filter rule.
 
@@ -160,7 +161,7 @@ class FirewallPolicyAPI(APIEndpoint):
             users (list): The IDs for the users that this rule applies to.
 
         Returns:
-            :obj:`dict`: The updated firewall filter rule resource record.
+            :obj:`Box`: The updated firewall filter rule resource record.
 
         Examples:
             Update the destination IP addresses for a rule:
@@ -191,7 +192,7 @@ class FirewallPolicyAPI(APIEndpoint):
 
         return self._put(f"firewallFilteringRules/{rule_id}", json=payload)
 
-    def delete_rule(self, rule_id: str):
+    def delete_rule(self, rule_id: str) -> int:
         """
         Deletes the specified firewall filter rule.
 
@@ -199,7 +200,7 @@ class FirewallPolicyAPI(APIEndpoint):
             rule_id (str): The unique identifier for the firewall filter rule.
 
         Returns:
-            :obj:`str`: The status code for the operation.
+            :obj:`int`: The status code for the operation.
 
         Examples:
             >>> zia.firewall.delete_rule('278454')
@@ -208,7 +209,7 @@ class FirewallPolicyAPI(APIEndpoint):
 
         return self._delete(f"firewallFilteringRules/{rule_id}", box=False).status_code
 
-    def list_ip_destination_groups(self, exclude_type: str = None):
+    def list_ip_destination_groups(self, exclude_type: str = None) -> BoxList:
         """
         Returns a list of IP Destination Groups.
 
@@ -217,7 +218,7 @@ class FirewallPolicyAPI(APIEndpoint):
                 Accepted values are: `DSTN_IP`, `DSTN_FQDN`, `DSTN_DOMAIN` and `DSTN_OTHER`.
 
         Returns:
-            :obj:`list` of :obj:`dict`: List of IP Destination Group records.
+            :obj:`BoxList`: List of IP Destination Group records.
 
         Examples:
             >>> for group in zia.firewall.list_ip_destination_groups():
@@ -227,9 +228,9 @@ class FirewallPolicyAPI(APIEndpoint):
 
         payload = {"excludeType": exclude_type}
 
-        return self._get("ipDestinationGroups", params=payload, box=BoxList)
+        return self._get("ipDestinationGroups", params=payload)
 
-    def get_ip_destination_group(self, group_id: str):
+    def get_ip_destination_group(self, group_id: str) -> Box:
         """
         Returns information on the specified IP Destination Group.
 
@@ -237,7 +238,7 @@ class FirewallPolicyAPI(APIEndpoint):
             group_id (str): The unique ID of the IP Destination Group.
 
         Returns:
-            :obj:`dict`: The IP Destination Group resource record.
+            :obj:`Box`: The IP Destination Group resource record.
 
         Examples:
             >>> pprint(zia.firewall.get_ip_destination_group('287342'))
@@ -245,7 +246,7 @@ class FirewallPolicyAPI(APIEndpoint):
         """
         return self._get(f"ipDestinationGroups/{group_id}")
 
-    def delete_ip_destination_group(self, group_id: str):
+    def delete_ip_destination_group(self, group_id: str) -> int:
         """
         Deletes the specified IP Destination Group.
 
@@ -253,7 +254,7 @@ class FirewallPolicyAPI(APIEndpoint):
             group_id (str): The unique ID of the IP Destination Group.
 
         Returns:
-            :obj:`str`: The status code of the operation.
+            :obj:`int`: The status code of the operation.
 
         Examples:
             >>> zia.firewall.delete_ip_destination_group('287342')
@@ -261,7 +262,7 @@ class FirewallPolicyAPI(APIEndpoint):
         """
         return self._delete(f"ipDestinationGroups/{group_id}", box=False).status_code
 
-    def add_ip_destination_group(self, name: str, **kwargs):
+    def add_ip_destination_group(self, name: str, **kwargs) -> Box:
         """
         Adds a new IP Destination Group.
 
@@ -277,7 +278,7 @@ class FirewallPolicyAPI(APIEndpoint):
             countries (list): Destination IP address counties.
 
         Returns:
-            :obj:`dict`: The newly created IP Destination Group resource record.
+            :obj:`Box`: The newly created IP Destination Group resource record.
 
         Examples:
             Add a Destination IP Group with IP addresses:
@@ -309,7 +310,7 @@ class FirewallPolicyAPI(APIEndpoint):
 
         return self._post("ipDestinationGroups", json=payload)
 
-    def update_ip_destination_group(self, group_id: str, **kwargs):
+    def update_ip_destination_group(self, group_id: str, **kwargs) -> Box:
         """
         Updates the specified IP Destination Group.
 
@@ -325,7 +326,7 @@ class FirewallPolicyAPI(APIEndpoint):
             countries (list): Destination IP address countries.
 
         Returns:
-            :obj:`dict`: The updated IP Destination Group resource record.
+            :obj:`Box`: The updated IP Destination Group resource record.
 
         Examples:
             Update the name of an IP Destination Group:
@@ -342,10 +343,7 @@ class FirewallPolicyAPI(APIEndpoint):
         """
 
         # Set payload to value of existing record
-        payload = {
-            snake_to_camel(k): v
-            for k, v in self.get_ip_destination_group(group_id).items()
-        }
+        payload = {snake_to_camel(k): v for k, v in self.get_ip_destination_group(group_id).items()}
 
         # Update payload
         for key, value in kwargs.items():
@@ -353,7 +351,7 @@ class FirewallPolicyAPI(APIEndpoint):
 
         return self._put(f"ipDestinationGroups/{group_id}", json=payload)
 
-    def list_ip_source_groups(self, search: str = None):
+    def list_ip_source_groups(self, search: str = None) -> BoxList:
         """
         Returns a list of IP Source Groups.
 
@@ -361,7 +359,7 @@ class FirewallPolicyAPI(APIEndpoint):
             search (str): The search string used to match against a group's name or description attributes.
 
         Returns:
-            :obj:`list` of :obj:`dict`: List of IP Source Group records.
+            :obj:`BoxList`: List of IP Source Group records.
 
         Examples:
             List all IP Source Groups:
@@ -378,9 +376,9 @@ class FirewallPolicyAPI(APIEndpoint):
 
         payload = {"search": search}
 
-        return self._get("ipSourceGroups", params=payload, box=BoxList)
+        return self._get("ipSourceGroups", params=payload)
 
-    def get_ip_source_group(self, group_id: str):
+    def get_ip_source_group(self, group_id: str) -> Box:
         """
         Returns information for the specified IP Source Group.
 
@@ -388,7 +386,7 @@ class FirewallPolicyAPI(APIEndpoint):
             group_id (str): The unique ID of the IP Source Group.
 
         Returns:
-            :obj:`dict`: The IP Source Group resource record.
+            :obj:`Box`: The IP Source Group resource record.
 
         Examples:
             >>> pprint(zia.firewall.get_ip_source_group('762398')
@@ -396,7 +394,7 @@ class FirewallPolicyAPI(APIEndpoint):
         """
         return self._get(f"ipSourceGroups/{group_id}")
 
-    def delete_ip_source_group(self, group_id: str):
+    def delete_ip_source_group(self, group_id: str) -> int:
         """
         Deletes an IP Source Group.
 
@@ -404,7 +402,7 @@ class FirewallPolicyAPI(APIEndpoint):
             group_id (str): The unique ID of the IP Source Group to be deleted.
 
         Returns:
-            :obj:`str`: The status code for the operation.
+            :obj:`int`: The status code for the operation.
 
         Examples:
             >>> zia.firewall.delete_ip_source_group('762398')
@@ -412,9 +410,7 @@ class FirewallPolicyAPI(APIEndpoint):
         """
         return self._delete(f"ipSourceGroups/{group_id}", box=False).status_code
 
-    def add_ip_source_group(
-            self, name: str, ip_addresses: list, description: str = None
-    ):
+    def add_ip_source_group(self, name: str, ip_addresses: list, description: str = None) -> Box:
         """
         Adds a new IP Source Group.
 
@@ -424,7 +420,7 @@ class FirewallPolicyAPI(APIEndpoint):
             description (str): Additional information for the IP Source Group.
 
         Returns:
-            :obj:`dict`: The new IP Source Group resource record.
+            :obj:`Box`: The new IP Source Group resource record.
 
         Examples:
             Add a new IP Source Group:
@@ -443,7 +439,7 @@ class FirewallPolicyAPI(APIEndpoint):
 
         return self._post("ipSourceGroups", json=payload)
 
-    def update_ip_source_group(self, group_id: str, **kwargs):
+    def update_ip_source_group(self, group_id: str, **kwargs) -> Box:
         """
         Update an IP Source Group.
 
@@ -459,7 +455,7 @@ class FirewallPolicyAPI(APIEndpoint):
             description (str): Additional information for the IP Source Group.
 
         Returns:
-            :obj:`dict`: The updated IP Source Group resource record.
+            :obj:`Box`: The updated IP Source Group resource record.
 
         Examples:
             Update the name of an IP Source Group:
@@ -476,9 +472,7 @@ class FirewallPolicyAPI(APIEndpoint):
         """
 
         # Set payload to value of existing record
-        payload = {
-            snake_to_camel(k): v for k, v in self.get_ip_source_group(group_id).items()
-        }
+        payload = {snake_to_camel(k): v for k, v in self.get_ip_source_group(group_id).items()}
 
         # Update payload
         for key, value in kwargs.items():
@@ -486,12 +480,12 @@ class FirewallPolicyAPI(APIEndpoint):
 
         return self._put(f"ipSourceGroups/{group_id}", json=payload)
 
-    def list_network_app_groups(self, search: str = None):
+    def list_network_app_groups(self, search: str = None) -> BoxList:
         """
         Returns a list of all Network Application Groups.
 
         Returns:
-            :obj:`list` of :obj:`dict`: The list of Network Application Group resource records.
+            :obj:`BoxList`: The list of Network Application Group resource records.
 
         Examples:
             >>> for group in zia.firewall.list_network_app_groups():
@@ -499,9 +493,9 @@ class FirewallPolicyAPI(APIEndpoint):
 
         """
         payload = {"search": search}
-        return self._get("networkApplicationGroups", params=payload, box=BoxList)
+        return self._get("networkApplicationGroups", params=payload)
 
-    def get_network_app_group(self, group_id: str):
+    def get_network_app_group(self, group_id: str) -> Box:
         """
         Returns information for the specified Network Application Group.
 
@@ -510,7 +504,7 @@ class FirewallPolicyAPI(APIEndpoint):
                 The unique ID for the Network Application Group.
 
         Returns:
-            :obj:`dict`: The Network Application Group resource record.
+            :obj:`Box`: The Network Application Group resource record.
 
         Examples:
             >>> pprint(zia.firewall.get_network_app_group('762398'))
@@ -518,7 +512,7 @@ class FirewallPolicyAPI(APIEndpoint):
         """
         return self._get(f"networkApplicationGroups/{group_id}")
 
-    def list_network_apps(self, search: str = None):
+    def list_network_apps(self, search: str = None) -> BoxList:
         """
         Returns a list of all predefined Network Applications.
 
@@ -526,7 +520,7 @@ class FirewallPolicyAPI(APIEndpoint):
             search (str): The search string used to match against a network application's description attribute.
 
         Returns:
-            :obj:`list` of :obj:`dict`: The list of Network Application resource records.
+            :obj:`BoxList`: The list of Network Application resource records.
 
         Examples:
             >>> for app in zia.firewall.list_network_apps():
@@ -534,9 +528,9 @@ class FirewallPolicyAPI(APIEndpoint):
 
         """
         payload = {"search": search}
-        return self._get("networkApplications", params=payload, box=BoxList)
+        return self._get("networkApplications", params=payload)
 
-    def get_network_app(self, app_id: str):
+    def get_network_app(self, app_id: str) -> Box:
         """
         Returns information for the specified Network Application.
 
@@ -544,7 +538,7 @@ class FirewallPolicyAPI(APIEndpoint):
             app_id (str): The unique ID for the Network Application.
 
         Returns:
-            :obj:`dict`: The Network Application resource record.
+            :obj:`Box`: The Network Application resource record.
 
         Examples:
             >>> pprint(zia.firewall.get_network_app('762398'))
@@ -552,7 +546,7 @@ class FirewallPolicyAPI(APIEndpoint):
         """
         return self._get(f"networkApplications/{app_id}")
 
-    def list_network_svc_groups(self, search: str = None):
+    def list_network_svc_groups(self, search: str = None) -> BoxList:
         """
         Returns a list of Network Service Groups.
 
@@ -560,7 +554,7 @@ class FirewallPolicyAPI(APIEndpoint):
             search (str):  The search string used to match against a group's name or description attributes.
 
         Returns:
-            :obj:`list` of :obj:`dict`: List of Network Service Group resource records.
+            :obj:`BoxList`: List of Network Service Group resource records.
 
         Examples:
             >>> for group in zia.firewall.list_network_svc_groups():
@@ -570,9 +564,9 @@ class FirewallPolicyAPI(APIEndpoint):
 
         payload = {"search": search}
 
-        return self._get("networkServiceGroups", params=payload, box=BoxList)
+        return self._get("networkServiceGroups", params=payload)
 
-    def get_network_svc_group(self, group_id: str):
+    def get_network_svc_group(self, group_id: str) -> Box:
         """
         Returns information for the specified Network Service Group.
 
@@ -580,7 +574,7 @@ class FirewallPolicyAPI(APIEndpoint):
             group_id (str): The unique ID for the Network Service Group.
 
         Returns:
-            :obj:`dict`: The Network Service Group resource record.
+            :obj:`Box`: The Network Service Group resource record.
 
         Examples:
             >>> pprint(zia.firewall.get_network_svc_group('762398'))
@@ -588,7 +582,7 @@ class FirewallPolicyAPI(APIEndpoint):
         """
         return self._get(f"networkServiceGroups/{group_id}")
 
-    def delete_network_svc_group(self, group_id: str):
+    def delete_network_svc_group(self, group_id: str) -> int:
         """
         Deletes the specified Network Service Group.
 
@@ -596,17 +590,15 @@ class FirewallPolicyAPI(APIEndpoint):
             group_id (str): The unique identifier for the Network Service Group.
 
         Returns:
-            :obj:`str`: The response code for the operation.
+            :obj:`int`: The response code for the operation.
 
         Examples:
             >>> zia.firewall.delete_network_svc_group('762398')
 
         """
-        return self._delete(f"networkServiceGroups/{group_id}")
+        return self._delete(f"networkServiceGroups/{group_id}", box=False).status_code
 
-    def add_network_svc_group(
-            self, name: str, service_ids: list, description: str = None
-    ):
+    def add_network_svc_group(self, name: str, service_ids: list, description: str = None) -> Box:
         """
         Adds a new Network Service Group.
 
@@ -616,7 +608,7 @@ class FirewallPolicyAPI(APIEndpoint):
             description (str): Additional information about the Network Service Group.
 
         Returns:
-            :obj:`dict`: The newly created Network Service Group resource record.
+            :obj:`Box`: The newly created Network Service Group resource record.
 
         Examples:
             Add a new Network Service Group:
@@ -634,7 +626,7 @@ class FirewallPolicyAPI(APIEndpoint):
 
         return self._post("networkServiceGroups", json=payload)
 
-    def list_network_services(self, search: str = None, protocol: str = None):
+    def list_network_services(self, search: str = None, protocol: str = None) -> BoxList:
         """
         Returns a list of all Network Services.
 
@@ -646,7 +638,7 @@ class FirewallPolicyAPI(APIEndpoint):
                 `GRE`, `ESP` and `OTHER`.
 
         Returns:
-            :obj:`list` of :obj:`dict`: The list of Network Service resource records.
+            :obj:`BoxList`: The list of Network Service resource records.
 
         Examples:
             >>> for service in zia.firewall.list_network_services():
@@ -654,9 +646,9 @@ class FirewallPolicyAPI(APIEndpoint):
 
         """
         payload = {"search": search, "protocol": protocol}
-        return self._get("networkServices", params=payload, box=BoxList)
+        return self._get("networkServices", params=payload)
 
-    def get_network_service(self, service_id: str):
+    def get_network_service(self, service_id: str) -> Box:
         """
         Returns information for the specified Network Service.
 
@@ -664,7 +656,7 @@ class FirewallPolicyAPI(APIEndpoint):
             service_id (str): The unique ID for the Network Service.
 
         Returns:
-            :obj:`dict`: The Network Service resource record.
+            :obj:`Box`: The Network Service resource record.
 
         Examples:
             >>> pprint(zia.firewall.get_network_service('762398'))
@@ -672,7 +664,7 @@ class FirewallPolicyAPI(APIEndpoint):
         """
         return self._get(f"networkServices/{service_id}")
 
-    def delete_network_service(self, service_id: str):
+    def delete_network_service(self, service_id: str) -> int:
         """
         Deletes the specified Network Service.
 
@@ -680,7 +672,7 @@ class FirewallPolicyAPI(APIEndpoint):
             service_id (str): The unique ID for the Network Service.
 
         Returns:
-            :obj:`str`: The status code for the operation.
+            :obj:`int`: The status code for the operation.
 
         Examples:
             >>> zia.firewall.delete_network_service('762398')
@@ -688,7 +680,7 @@ class FirewallPolicyAPI(APIEndpoint):
         """
         return self._delete(f"networkServices/{service_id}", box=False).status_code
 
-    def add_network_service(self, name: str, ports: list = None, **kwargs):
+    def add_network_service(self, name: str, ports: list = None, **kwargs) -> Box:
         """
         Adds a new Network Service.
 
@@ -712,7 +704,7 @@ class FirewallPolicyAPI(APIEndpoint):
             description (str): Additional information on the Network Service.
 
         Returns:
-            :obj:`dict`: The newly created Network Service resource record.
+            :obj:`Box`: The newly created Network Service resource record.
 
         Examples:
             Add Network Service for Microsoft Exchange:
@@ -740,12 +732,10 @@ class FirewallPolicyAPI(APIEndpoint):
         # Convert tuple list to dict and add to payload
         if ports is not None:
             for items in ports:
-                port_obj = {"start": items[2]}
-                if len(ports) == 4:
-                    port_obj["end"] = items[3]
-                payload.setdefault(f"{items[0]}{items[1].title()}Ports", []).append(
-                    port_obj
-                )
+                port_range = [{"start": items[2]}]
+                if len(items) == 4:
+                    port_range.append({"end": items[3]})
+                payload.setdefault(f"{items[0]}{items[1].title()}Ports", []).extend(port_range)
 
         # Add optional parameters to payload
         for key, value in kwargs.items():
@@ -753,7 +743,7 @@ class FirewallPolicyAPI(APIEndpoint):
 
         return self._post("networkServices", json=payload)
 
-    def update_network_service(self, service_id: str, ports: list = None, **kwargs):
+    def update_network_service(self, service_id: str, ports: list = None, **kwargs) -> Box:
         """
         Updates the specified Network Service.
 
@@ -779,7 +769,7 @@ class FirewallPolicyAPI(APIEndpoint):
             description (str): Additional information on the Network Service.
 
         Returns:
-            :obj:`dict`: The newly created Network Service resource record.
+            :obj:`Box`: The newly created Network Service resource record.
 
         Examples:
             Update the name and description for a Network Service:
@@ -796,21 +786,17 @@ class FirewallPolicyAPI(APIEndpoint):
 
 
         """
-        existing_service = self.get_network_service(service_id)
+        payload = {snake_to_camel(k): v for k, v in self.get_network_service(service_id).items()}
 
         # Convert tuple list to dict and add to payload
         if ports is not None:
-            payload = {}
+            # Clear existing ports and set new values
             for items in ports:
-                port_obj = {"start": items[2]}
-                if len(ports) == 4:
-                    port_obj["end"] = items[3]
-                payload.setdefault(f"{items[0]}{items[1].title()}Ports", []).append(
-                    port_obj
-                )
-        else:
-            # Use existing values and convert back to camelCase
-            payload = {snake_to_camel(k): v for k, v in existing_service.items()}
+                port_key = f"{items[0]}{items[1].title()}Ports"
+                payload[port_key] = []
+                payload[port_key].append({"start": items[2]})
+                if len(items) == 4:
+                    payload[port_key].append({"end": items[3]})
 
         # Add optional parameters to payload
         for key, value in kwargs.items():
