@@ -23,7 +23,7 @@ class LSSConfigControllerAPI(APIEndpoint):
         self.v2_url = api.v2_url
         self.v2_admin_url = "https://config.private.zscaler.com/mgmtconfig/v2/admin/lssConfig"
 
-    def _create_policy(self, conditions) -> list:
+    def _create_policy(self, conditions: list) -> list:
         """
         Creates a dict template for feeding conditions into the ZPA Policies API when adding or updating a policy.
 
